// en Scala
package calc;

abstract class Expr {
  /** position pour signaler les erreurs, presente dans tous les noeuds. */
  def pos: Int;
  // il faut mettre `override' et `()', parce qu'on herite une method `toString()' de java.lang.Object 
  override def toString(): String ; 
  def eval: Int;
}
/** sous-classe pour representer les nombres */
case class IntLit(pos: int, value: int) extends Expr {
  override def toString(): String = value.toString();
  def eval: Int = value;
}
// le champ "pos" de IntLit definit automatiquement la methode "pos" de Expr.

/** sous-classe pour representer les operations */
case class BinOp(pos: Int, op: int, left: Expr, right: Expr) extends Expr {
  override def toString(): String =  
    "("+left.toString()+Scanner.representation(op)+right.toString()+")";
  def eval: Int = {
      val l: int = left.eval;
      val r: int = right.eval;
      op match {
        case Tokens.PLUS  => return l+r;
        case Tokens.MINUS => return l-r;
        case Tokens.MUL   => return l*r;
        case Tokens.DIV   => return l/r;
        case _             => throw new InternalError();
      }
  }
}
