// en Java
package calc;

public abstract class Expr {
  /** position pour signaler les erreurs, presente dans tous les noeuds. */
  int pos;

  public abstract String toString();
  public abstract int eval();

  /** sous-classe pour representer les nombres */
  static class IntLit extends Expr {
    int value;
    IntLit(int pos, int value) {
      this.pos = pos;
      this.value = value;
    }
    public String toString() {
      return String.valueOf(value);
    }
    public int eval() {
      return value;
    }
  }

  /** sous-classe pour representer les operations */
  static class BinOp extends Expr {
    int operator;
    Expr left, right;

    BinOp(int pos, int operator, Expr left, Expr right) {
      this.pos = pos;
      this.operator = operator;
      this.left = left;
      this.right = right;
    }

    public String toString() {
      return "("
        +left.toString()
        +Scanner.representation(operator)
        +right.toString()+")";
    }
    public int eval() {
      int l = left.eval();
      int r = right.eval();
      switch(operator) {
        case Scanner.PLUS:  return l+r;
        case Scanner.MINUS: return l-r;
        case Scanner.MUL:   return l*r;
        case Scanner.DIV:   return l/r;
        default: throw new InternalError();
      }
    }
  }
}
