//
// Arbres visitable pour les expressions
//
// en Java
package calc;

public abstract class Expr {
    /** position pour signaler les erreurs, presente dans tous les noeuds. */
    int pos;
    public abstract void apply(Visitor v);

    /** sous-classe pour representer les nombres */
    static class IntLit extends Expr {
        int value;
        public void apply(Visitor v) {
            v.caseIntLit(this);
        }

        IntLit(int pos, int value) {
            this.pos = pos;
            this.value = value;
        }
    }
    
    /** sous-classe pour representer les operations */
    static class BinOp extends Expr {
        int operator;
        Expr left, right;
        public void apply(Visitor v) {
            v.caseBinOp(this);
        }

        BinOp(int pos, int operator, Expr left, Expr right) {
            this.pos = pos;
            this.operator = operator;
            this.left = left;
            this.right = right;
        }
    }
    
    public interface Visitor {
        public void caseIntLit(IntLit tree);
        public void caseBinOp(BinOp tree);
    }
}


